'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var css = require('css');

var _require = require('glamor'),
    glamorStyleSheet = _require.styleSheet;

var _require2 = require('./replace-class-names'),
    replaceClassNames = _require2.replaceClassNames;

function createSerializer(styleSheet) {
  function test(val) {
    return val && !val.withStyles && val.$$typeof === Symbol.for('react.test.json');
  }

  function print(val, printer) {
    var selectors = getSelectors(val);
    var styles = getStyles(selectors);
    val.withStyles = true;
    var printedVal = printer(val);
    if (styles) {
      return replaceClassNames(selectors, styles, printedVal);
    } else {
      return printedVal;
    }
  }

  function getSelectors(node) {
    var selectors = [];
    if (node.children && node.children.reduce) {
      selectors = node.children.reduce(function (acc, child) {
        return acc.concat(getSelectors(child));
      }, []);
    }
    if (node.props) {
      return getSelectorsFromProps(selectors, node.props);
    }
    return selectors;
  }

  function getSelectorsFromProps(selectors, props) {
    var className = props.className || props.class;
    if (className) {
      selectors = selectors.concat(className.toString().split(' ').map(function (cn) {
        return `.${cn}`;
      }));
    }
    var dataProps = Object.keys(props).reduce(function (dProps, key) {
      if (key.startsWith('data-')) {
        dProps.push(`[${key}]`);
      }
      return dProps;
    }, []);
    if (dataProps.length) {
      selectors = selectors.concat(dataProps);
    }
    return selectors;
  }

  function getStyles(nodeSelectors) {
    var styles = styleSheet.tags.map(function (tag) {
      return (/* istanbul ignore next */tag.textContent || ''
      );
    }).join('\n');
    var ast = css.parse(styles);
    var rules = ast.stylesheet.rules.filter(filter);
    var mediaQueries = getMediaQueries(ast, filter);

    ast.stylesheet.rules = [].concat(_toConsumableArray(rules), _toConsumableArray(mediaQueries));

    var ret = css.stringify(ast);
    return ret;

    function filter(rule) {
      if (rule.type === 'rule') {
        return rule.selectors.some(function (selector) {
          var baseSelector = selector.split(/:| /)[0];
          return nodeSelectors.includes(baseSelector);
        });
      }
      return false;
    }
  }

  function getMediaQueries(ast, filter) {
    return ast.stylesheet.rules.filter(function (rule) {
      return rule.type === 'media';
    }).reduce(function (acc, mediaQuery) {
      mediaQuery.rules = mediaQuery.rules.filter(filter);

      if (mediaQuery.rules.length) {
        return acc.concat(mediaQuery);
      }

      return acc;
    }, []);
  }
  return { test, print };
}

var glamorSerializer = createSerializer(glamorStyleSheet);
createSerializer.test = glamorSerializer.test;
createSerializer.print = glamorSerializer.print;

module.exports = createSerializer;